using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using Weland;
using Gtk;
using System.Drawing;

namespace Resize
{
	public class Plugin
	{

		public static bool Compatible()
		{
			return true;
		}

		public static string Name()
		{
			return "Resize Level";
		}

		public static void GtkRun(Editor editor)
		{
			new Resize(editor);
		}
	}

	class Resize : Window
	{

		Editor _editor;
		Entry txtPercent;
		RadioButton rbnGrow;

		public Resize(Editor editor) : base("Resize")
		{
			_editor = editor;

			// Horrible GTK code
			SetDefaultSize(250, 200);
			SetPosition(WindowPosition.Center);
			this.BorderWidth = 0;

			VBox box1 = new VBox(false, 0);
			this.Add(box1);
			box1.Show();

			VBox box2 = new VBox(false, 10);
			box2.BorderWidth = 3;
			box1.PackStart(box2, true, true, 0);
			box2.Show();

			rbnGrow = new RadioButton(null, "Grow Level");
			rbnGrow.Active = true;
			box2.PackStart(rbnGrow, true, true, 0);
			rbnGrow.Show();

			RadioButton rbnShrink = new RadioButton(rbnGrow, "Shrink Level");
			box2.PackStart(rbnShrink, true, true, 0);
			rbnShrink.Show();

			VBox box3 = new VBox(false, 10);
			box3.BorderWidth = 10;
			box1.PackStart(box3, false, true, 0);
			box3.Show();

			Label lblPercent = new Label();
			lblPercent.Text = "Percentage to grow/shrink";
			box3.PackStart(lblPercent, true, true, 0);

			VBox box4 = new VBox(false, 10);
			box4.BorderWidth = 10;
			box1.PackStart(box4, false, true, 0);
			box4.Show();

			txtPercent = new Entry();
			box4.PackStart(txtPercent, true, true, 0);

			VBox box5 = new VBox(false, 10);
			box5.BorderWidth = 10;
			box1.PackStart(box5, false, true, 0);
			box5.Show();

			Button button = new Button("Resize");
			button.Clicked += resize_event;

			box5.PackStart(button, true, true, 0);
			button.CanDefault = true;
			button.GrabDefault();
			button.Show();

			txtPercent.GrabFocus();

			this.ShowAll();
		}

		void resize_event(object obj, EventArgs args)
		{
			try
			{
				double percent = Convert.ToDouble(txtPercent.Text);
				percent /= 100;
				if (!rbnGrow.Active)
					percent *= -1;
				ResizeLevel(_editor, percent);
				this.Destroy();
			}
			catch
			{ // Having a prahblum?
				MessageDialog md = new MessageDialog(null, DialogFlags.Modal, MessageType.Info, ButtonsType.Ok,
									  "Prahblum detected!\n\nCan't grow/shrink that much.  Try a smaller percentage.\n\n" +
									  "Or perhaps you didn't type in a number, ASS-HOLE?");
				md.Run();
				md.Destroy();
				_editor.Undo();
			}
		}

		private void ResizeLevel(Editor editor, double scale)
		{
			Level level = editor.Level;

			// Start with a recenter to make sure we can embiggen the map as much as possible.
			editor.Recenter();

			int offsetX, offsetY;

			// Shrink/expand
			for (short i = 0; i < level.Endpoints.Count; ++i)
			{
				offsetX = (int)(level.Endpoints[i].X * scale);
				offsetY = (int)(level.Endpoints[i].Y * scale);

				// Make sure we don't grow the level past the max level size
				if (level.Endpoints[i].X + offsetX > short.MaxValue ||
				  level.Endpoints[i].Y + offsetY > short.MaxValue ||
				  level.Endpoints[i].X + offsetX < short.MinValue ||
				  level.Endpoints[i].Y + offsetY < short.MinValue)
				{
					throw new Exception("Nonononono");
				}

				TranslatePoint(editor, i, offsetX, offsetY);
			}

			foreach (MapObject o in level.Objects)
			{
				offsetX = (int)(o.X * scale);
				offsetY = (int)(o.Y * scale);
				TranslateObject(o, offsetX, offsetY);
			}

			foreach (Annotation a in level.Annotations)
			{
				offsetX = (int)(a.X * scale);
				offsetY = (int)(a.Y * scale);
				TranslateAnnotation(a, offsetX, offsetY);
			}

			editor.Changed = true;
		}

		public void TranslatePoint(Editor editor, short index, int X, int Y)
		{
			Weland.Point p = editor.Level.Endpoints[index];
			short x = (short)(p.X + X);
			short y = (short)(p.Y + Y);
			MovePoint(editor, index, x, y);
		}

		void MovePoint(Editor editor, short index, short X, short Y)
		{
			Weland.Point p = editor.Level.Endpoints[index];
			p.X = X;
			p.Y = Y;
			editor.Level.Endpoints[index] = p;

			// update attached polygon concavity
			List<short> polygons = editor.Level.FindEndpointPolygons(index);

			foreach (short i in polygons)
			{
				Polygon polygon = editor.Level.Polygons[i];
				editor.Level.UpdatePolygonConcavity(polygon);
			}
		}

		public void TranslateObject(MapObject obj, int X, int Y)
		{
			obj.X = (short)(obj.X + X);
			obj.Y = (short)(obj.Y + Y);
		}

		public void TranslateAnnotation(Annotation a, int X, int Y)
		{
			a.X = (short)(a.X + X);
			a.Y = (short)(a.Y + Y);
		}
	}
}