﻿using System;
using System.Collections.Generic;
using GeneratorCommon;
using Weland;

namespace Texturer
{
	public class Plugin
	{
		private static byte _collection;
		private static byte[] _floorTextures;
		private static byte[] _ceilingTextures;
		private static byte[] _wallTextures;
		private static byte _landscapeCollection;

		public static bool Compatible()
		{
			return true;
		}

		public static string Name()
		{
			return "Super Pave";
		}

		public static void Run(Editor editor)
		{
			try
			{
				_collection = Helpers.GetRandomCollection();

				_floorTextures = Helpers.GetRandomizedFloorTextures(_collection);
				_ceilingTextures = Helpers.GetRandomizedFloorTextures(_collection);
				_wallTextures = Helpers.GetRandomizedWallTextures(_collection);
				_landscapeCollection = Helpers.GetRandomLandscapeCollection();

				var h = new Helpers(editor);
				TextureLevel(h, editor);
				ShadeLevel(h);
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex);
			}
		}

		private static void TextureLevel(Helpers h, Editor editor)
		{
			// Set up groups of polygons based on floor/ceiling heights
			var floorMapping = new Dictionary<double, List<Polygon>>();
			var ceilingMapping = new Dictionary<double, List<Polygon>>();
			var stairMapping = new Dictionary<string, List<Polygon>>();
			foreach (var poly in editor.Level.Polygons)
			{
				if (!floorMapping.ContainsKey(poly.FloorHeight))
					floorMapping[poly.FloorHeight] = new List<Polygon>();

				floorMapping[poly.FloorHeight].Add(poly);

				if (!ceilingMapping.ContainsKey(poly.CeilingHeight))
					ceilingMapping[poly.CeilingHeight] = new List<Polygon>();

				ceilingMapping[poly.CeilingHeight].Add(poly);
			}

			// Set up groups of polygons based on stair annotations
			var stairAnnotations = new List<Annotation>();
			foreach (var a in editor.Level.Annotations)
			{
				if (a.Text.StartsWith("ST_"))
				{
					if (!stairMapping.ContainsKey(a.Text))
						stairMapping[a.Text] = new List<Polygon>();

					stairMapping[a.Text].Add(editor.Level.Polygons[a.PolygonIndex]);
					stairAnnotations.Add(a);
				}
			}

			// Probably should leave these so it is re-runnable.
			// foreach (var a in stairAnnotations)
			// {
			// 	editor.Level.Annotations.Remove(a);
			// }

			foreach (var key in floorMapping.Keys)
			{
				var rand = Helpers.Rand(0, _floorTextures.Length);
				var textureIndex = _floorTextures[rand];
				rand = Helpers.Rand(0, _wallTextures.Length);
				var wallTextureIndex = _wallTextures[rand];
				var lightIndex = Helpers.Rand(0, 21);

				foreach (var poly in floorMapping[key])
				{
					Helpers.SetFloorTexture(poly, _collection, textureIndex);
					poly.FloorLight = lightIndex;
					h.SetWallTextures(poly, _collection, wallTextureIndex);
				}
			}

			foreach (var key in ceilingMapping.Keys)
			{
				var rand = Helpers.Rand(0, _ceilingTextures.Length);
				var textureIndex = _ceilingTextures[rand];
				var lightIndex = Helpers.Rand(0, 21);

				foreach (var poly in ceilingMapping[key])
				{
					Helpers.SetCeilingTexture(poly, _collection, textureIndex);
					poly.CeilingLight = lightIndex;
				}
			}

			foreach (var key in stairMapping.Keys)
			{
				var rand = Helpers.Rand(0, _floorTextures.Length);
				var textureIndex = _floorTextures[rand];
				rand = Helpers.Rand(0, _wallTextures.Length);
				var wallTextureIndex = _wallTextures[rand];
				rand = Helpers.Rand(0, _wallTextures.Length);
				//var secondaryWallTextureIndex = _wallTextures[rand];
				var lightIndex = Helpers.Rand(0, 21);

				foreach (var poly in stairMapping[key])
				{
					Helpers.SetFloorTexture(poly, _collection, textureIndex);
					poly.FloorLight = lightIndex;
					h.SetWallTextures(poly, _collection, wallTextureIndex);
					//h.SetWallTextures(poly, _collection, secondaryWallTextureIndex, true, true);
					h.SetWallTextures(poly, _collection, wallTextureIndex, true, true);
				}
			}

			// Landscape textures

			// Find highest ceiling height
			var highest = double.MinValue;
			foreach (var height in ceilingMapping.Keys)
			{
				if (height > highest)
					highest = height;
			}

			foreach (var poly in ceilingMapping[highest])
			{
				Helpers.SetCeilingLandscapeTexture(poly, _landscapeCollection);
			}

			h.TextureTeleporters(_collection);
			h.TexturePlatforms(_collection);
		}

		private static void ShadeLevel(Helpers h)
		{
			short highlight = 5;
			short shadow = 17;

			h.AutoShade(highlight, shadow);
		}
	}
}
